@@ -938,7 +938,11 @@ var LibraryEmbind = {
   // (hand-written JS code) -> (autogenerated JS invoker) -> (template-generated C++ invoker) -> (target C++ function)
   // craftInvokerFunction generates the JS invoker function for each function exposed to JS through embind.
   $craftInvokerFunction__deps: [
-    '$makeLegalFunctionName', '$new_', '$runDestructors', '$throwBindingError'],
+    '$makeLegalFunctionName', '$new_', '$runDestructors', '$throwBindingError',
+  #if ASYNCIFY
+    '$Asyncify',
+  #endif
+  ],
   $craftInvokerFunction: function(humanName, argTypes, classType, cppInvokerFunc, cppTargetFunc) {
     // humanName: a human-readable string name for the function to be generated.
     // argTypes: An array that contains the embind type objects for all types in the function signature.
@@ -1005,24 +1009,34 @@ var LibraryEmbind = {
 
       var rv = cppInvokerFunc.apply(null, invokerFuncArgs);
 
-      if (needsDestructorStack) {
-        runDestructors(destructors);
-      } else {
-        for (var i = isClassMethodFunc ? 1 : 2; i < argTypes.length; i++) {
-          var param = i === 1 ? thisWired : argsWired[i - 2];
-          if (argTypes[i].destructorFunction !== null) {
-            argTypes[i].destructorFunction(param);
+      function onDone(rv) {
+        if (needsDestructorStack) {
+          runDestructors(destructors);
+        } else {
+          for (var i = isClassMethodFunc ? 1 : 2; i < argTypes.length; i++) {
+            var param = i === 1 ? thisWired : argsWired[i - 2];
+            if (argTypes[i].destructorFunction !== null) {
+              argTypes[i].destructorFunction(param);
+            }
           }
         }
+
+  #if EMSCRIPTEN_TRACING
+        Module.emscripten_trace_exit_context();
+  #endif
+
+        if (returns) {
+          return argTypes[0].fromWireType(rv);
+        }
       }
 
-#if EMSCRIPTEN_TRACING
-      Module.emscripten_trace_exit_context();
+#if ASYNCIFY
+      if (Asyncify.currData) {
+        return Asyncify.whenDone().then(onDone);
+      }
 #endif
 
-      if (returns) {
-        return argTypes[0].fromWireType(rv);
-      }
+      return onDone(rv);
     };
 #else
     var argsList = "";
@@ -1073,6 +1087,12 @@ var LibraryEmbind = {
     invokerFnBody +=
         (returns?"var rv = ":"") + "invoker(fn"+(argsListWired.length>0?", ":"")+argsListWired+");\n";
 
+#if ASYNCIFY
+    args1.push("Asyncify");
+    args2.push(Asyncify);
+    invokerFnBody += "function onDone(rv) {\n";
+#endif
+
     if (needsDestructorStack) {
         invokerFnBody += "runDestructors(destructors);\n";
     } else {
@@ -1097,6 +1117,12 @@ var LibraryEmbind = {
         invokerFnBody += "Module.emscripten_trace_exit_context();\n";
 #endif
     }
+
+#if ASYNCIFY
+    invokerFnBody += "}\n";
+    invokerFnBody += "return Asyncify.currData ? Asyncify.whenDone().then(onDone) : onDone(rv);\n"
+#endif
+
     invokerFnBody += "}\n";
 
     args1.push(invokerFnBody);
@@ -2028,7 +2054,7 @@ var LibraryEmbind = {
 
   _embind_register_class_constructor__deps: [
     '$heap32VectorToArray', '$embind__requireFunction', '$runDestructors',
-    '$throwBindingError', '$whenDependentTypesAreResolved'],
+    '$throwBindingError', '$whenDependentTypesAreResolved', '$registeredTypes'],
   _embind_register_class_constructor: function(
     rawClassType,
     argCount,
@@ -2058,21 +2084,9 @@ var LibraryEmbind = {
         };
 
         whenDependentTypesAreResolved([], rawArgTypes, function(argTypes) {
-            classType.registeredClass.constructor_body[argCount - 1] = function constructor_body() {
-                if (arguments.length !== argCount - 1) {
-                    throwBindingError(humanName + ' called with ' + arguments.length + ' arguments, expected ' + (argCount-1));
-                }
-                destructors.length = 0;
-                args.length = argCount;
-                for (var i = 1; i < argCount; ++i) {
-                    args[i] = argTypes[i]['toWireType'](destructors, arguments[i - 1]);
-                }
-
-                var ptr = invoker.apply(null, args);
-                runDestructors(destructors);
-
-                return argTypes[0]['fromWireType'](ptr);
-            };
+            // Insert empty slot for context type (argTypes[1]).
+            argTypes.splice(1, 0, null);
+            classType.registeredClass.constructor_body[argCount - 1] = craftInvokerFunction(humanName, argTypes, null, invoker, rawConstructor);
             return [];
         });
         return [];
