@@ -8,7 +8,7 @@ mergeInto(LibraryManager.library, {
   $PIPEFS__postset: function() {
     addAtInit('PIPEFS.root = FS.mount(PIPEFS, {}, null);');
   },
-  $PIPEFS__deps: ['$FS', '$ERRNO_CODES'],
+  $PIPEFS__deps: ['$FS'],
   $PIPEFS: {
     BUCKET_BUFFER_SIZE: 1024 * 8, // 8KiB Buffer
     mount: function (mount) {
@@ -78,10 +78,10 @@ mergeInto(LibraryManager.library, {
         return 0;
       },
       ioctl: function (stream, request, varargs) {
-        return ERRNO_CODES.EINVAL;
+        return {{{ cDefine('EINVAL') }}};
       },
       fsync: function (stream) {
-        return ERRNO_CODES.EINVAL;
+        return {{{ cDefine('EINVAL') }}};
       },
       read: function (stream, buffer, offset, length, position /* ignored */) {
         var pipe = stream.node.pipe;
@@ -104,7 +104,7 @@ mergeInto(LibraryManager.library, {
         }
         if (currentLength == 0) {
           // Behave as if the read end is always non-blocking
-          throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
+          throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
         }
         var toRead = Math.min(currentLength, length);
 
